From b9b4d486a37cdca4b3a70c9fd8267c85f4679760 Mon Sep 17 00:00:00 2001
From: fincs <fincs.alt1@gmail.com>
Date: Fri, 18 Jan 2019 00:38:47 +0100
Subject: [PATCH 1/2] Fix GPUCMD_Add with more than 256 words (thanks to
 iMackshun for testing)

---
 libctru/source/gpu/gpu.c | 19 ++++++++++++++++---
 1 file changed, 16 insertions(+), 3 deletions(-)

diff --git a/libctru/source/gpu/gpu.c b/libctru/source/gpu/gpu.c
index 3b7afd2..02cea77 100644
--- a/libctru/source/gpu/gpu.c
+++ b/libctru/source/gpu/gpu.c
@@ -22,14 +22,13 @@ void GPUCMD_AddRawCommands(const u32* cmd, u32 size)
 	gpuCmdBufOffset+=size;
 }
 
-void GPUCMD_Add(u32 header, const u32* param, u32 paramlength)
+static void GPUCMD_AddInternal(u32 header, const u32* param, u32 paramlength)
 {
-	if(!paramlength)paramlength=1;
 	if(!gpuCmdBuf || gpuCmdBufOffset+paramlength+1>gpuCmdBufSize)
 		svcBreak(USERBREAK_PANIC); // Shouldn't happen.
 
 	paramlength--;
-	header|=(paramlength&0x7ff)<<20;
+	header|=(paramlength&0xff)<<20;
 
 	gpuCmdBuf[gpuCmdBufOffset]=param ? param[0] : 0;
 	gpuCmdBuf[gpuCmdBufOffset+1]=header;
@@ -45,6 +44,20 @@ void GPUCMD_Add(u32 header, const u32* param, u32 paramlength)
 	if(paramlength&1)gpuCmdBuf[gpuCmdBufOffset++]=0x00000000; //alignment
 }
 
+void GPUCMD_Add(u32 header, const u32* param, u32 paramlength)
+{
+	if(!paramlength)paramlength=1;
+
+	while(paramlength)
+	{
+		u32 remaining = paramlength > 0x100 ? 0x100 : paramlength;
+		GPUCMD_AddInternal(header, param, remaining);
+		param += remaining;
+		paramlength -= remaining;
+		if(header & BIT(31)) header += remaining;
+	}
+}
+
 void GPUCMD_Split(u32** addr, u32* size)
 {
 	GPUCMD_AddWrite(GPUREG_FINALIZE, 0x12345678);
-- 
2.20.1.windows.1

From 79a77cd86244692d7aeaa03bb42d09a7ad99639c Mon Sep 17 00:00:00 2001
From: Dave Murphy <davem@devkitpro.org>
Date: Fri, 18 Jan 2019 15:23:53 +0000
Subject: [PATCH 2/2] add SO_BROADCAST

---
 libctru/include/sys/socket.h | 2 ++
 1 file changed, 2 insertions(+)

diff --git a/libctru/include/sys/socket.h b/libctru/include/sys/socket.h
index f117153..8f6d78e 100644
--- a/libctru/include/sys/socket.h
+++ b/libctru/include/sys/socket.h
@@ -42,6 +42,8 @@
 #define SO_TYPE         0x1008      // get socket type
 #define SO_ERROR        0x1009      // get socket error
 
+#define SO_BROADCAST    0x0000      // unrequired, included for compatibility
+
 typedef uint32_t socklen_t;
 typedef uint16_t sa_family_t;
 
-- 
2.20.1.windows.1

